import json
import requests
import pandas as pd
from pathlib import Path
from src.movienight.datasets._config import tmdb_api_key

def _get_tmdb(
    url=["https://api.themoviedb.org/3/movie/popular","https://api.themoviedb.org/3/movie/top_rated"],
    api_key = tmdb_api_key,
    n_pages = 100,
    col_names = [
        "id",
        "title",
        "overview",
        "adult",
        "genre_ids",
        "release_date"
    ]
):

    # empty dataframe to populate
    movie_data = pd.DataFrame()

    # looping through endpoints and pages
    for i in url:
        for j in range(n_pages):
            page_num = j+1
            payload = {
                "api_key" : api_key,
                "region" : "US",
                "page" : page_num
            }
            r = requests.get(i, params=payload)
            results = r.json()
            movie_data = pd.concat([movie_data, pd.json_normalize(results['results'])[col_names]])\
                .drop_duplicates(subset='id')\
                .reset_index(drop=True)
    return movie_data

if __name__ == "__main__":
    filepath_ = Path("src/movie_finder/datasets/data/tmdb.csv")
    data_ = _get_tmdb()
    data_.to_csv(filepath_, index=False)


from sentence_transformers import SentenceTransformer, util

class MovieFinder():
    """
    MovieFinder
    
    Uses NLP to find movies based on a user-provided query's similarity to a movie's synopsis

    Parameters
    -----------
    sentence_transformer_model : str, default = 'all-MiniLM-L6-v2'
        Choose a pre-trained model from the sentence_transformers library.
    
    Attributes
    -----------
    model_ : NLP model from sentence_transformers library
    query_embedding_ : word embedding of provided query
    top_matches : top n movie matches based on similarity to query
      
    """
    def __init__(self, sentence_transformer_model = 'all-MiniLM-L6-v2'):
        self.model_ = SentenceTransformer(sentence_transformer_model)
        return
    def get_n_matches(self, query, data, col="overview", n=25):
        query_embedding_ = self.model_.encode(query)
        similarity_score = data.apply(lambda row: util.cos_sim(query_embedding_, self.model_.encode(row[col])).item(), axis=1)
        self.top_n_index = (-similarity_score).argsort()[:n]
        return


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['movienight', 'movienight.datasets']

package_data = \
{'': ['*'], 'movienight.datasets': ['data/*']}

install_requires = \
['dash>=2.8.1,<3.0.0',
 'numpy>=1.24.2,<2.0.0',
 'pandas>=1.5.3,<2.0.0',
 'pathlib>=1.0.1,<2.0.0',
 'plotly>=5.13.1,<6.0.0',
 'sentence-transformers>=2.2.2,<3.0.0']

setup_kwargs = {
    'name': 'movienight',
    'version': '0.1.0',
    'description': '',
    'long_description': 'Movie Night\n============\n\n**movienight** is a python package that uses NLP to return a list of recommended movies based on free form text input. **movienight** is not intended to be that useful and is more of a personal project.\n\n    ├──src/movienight/\n    │   └── _MovieNight.py           <- Holds all classes and functions relevant to MovieNigh\n    │       └── datasets/        \n    │           ├── _base.py         <- Utility function to load tmdb dataset\n    │           ├── _tmdb.py         <- Script used to query TMDB api\n    │           ├── _config.py       <- Config file that holds TMDB api key. Fill in with your own api key.\n    │           └── data             <- Holds datasets used by MovieNight\n    ├──MovieFinderApp/\n    │   └── movie\n    ├── demo.ipynb                   <- Notebook used to test MovieNight functionality\n    └── README.md\n\nQuick Start\n------------\nThis project uses ```poetry``` to manage dependencies. Download ```poetry``` following the instructions found [here](https://python-poetry.org/docs/), then navigate to the directory that contains the ```pyproject.toml``` file and run:\n\n```\npoetry install\n```',
    'author': 'dan-kwon',
    'author_email': 'danielkwon02@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import pandas as pd
from pathlib import Path

def load_tmdb():
    """
    """
    filepath_ = Path("src/movie_finder/datasets/data/tmdb.csv")
    data_ = pd.read_csv(
        filepath_, 
        dtype={
            "id" : int,
            "title" : str,
            "overview" : str,
            "adult": bool,
            "genre_ids" : object,
            "release_date" : str}
        )
    data_["release_date"] = pd.to_datetime(data_["release_date"])
    data_ = data_[data_.overview.notnull()]
    return data_
    